"""
grammar.py (c) 2011 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

from pyparsing import ( Optional,
	Dict, Literal, Word, Group,
	OneOrMore, Keyword,
	LineEnd, White, nums,
	CaselessLiteral, Combine, Or,
	Regex, Each, ParseFatalException)


def index_check(minval=0):
	def index_check_parseaction(string, loc, tokens):
		parsedval = tokens[0]
		if parsedval < minval:
			raise ParseFatalException(string, loc, 
					"index %d must be >=0" % (parsedval))
		return parsedval
	return index_check_parseaction


def instance():
	lit_e = CaselessLiteral('E')
	plusorminus = Literal('+') | Literal('-')
	number = Word(nums) 
	integer = Combine(Optional(plusorminus) +
						number).setParseAction(lambda t:int(t[0]))
	index = integer.copy().addParseAction(index_check(0))
	floatnumber = Combine( integer +
							Optional( Literal('.') + Optional(number) ) +
							Optional( lit_e + integer )
						).setParseAction(lambda t:float(t[0]))

	#comment = Suppress("%") + Word(alphanums + " ") 
	comment = Regex(r"%.*").setName("comment").suppress()
	linend = Or( [comment , LineEnd()] ).suppress()
	section_end = (Literal('#') + LineEnd()).suppress()

	vertex = (Group( OneOrMore( floatnumber('point') +
					 OneOrMore( White() ).suppress() ) ) + linend)('vertex')
	vertex_header = (Keyword('VERTEX') + linend).suppress()
	vertex_section = (vertex_header + Group(OneOrMore(vertex))('vertices') +
						section_end)

	simplex = (Group( OneOrMore( index('index')
					+ OneOrMore( White() ).suppress() ) ) + linend)('simplex')
	simplex_header = (Keyword('SIMPLEX') + linend).suppress()
	simplex_section = (simplex_header + Group(OneOrMore(simplex))('simplices') +
						section_end)

	boundarysegment = (Group( index('id') +
								OneOrMore( index('index') +
								OneOrMore( White() ).suppress() ) ) +
								linend)('boundarysegment')
	boundarysegment_header = (Keyword('BOUNDARYSEGMENTS') + linend).suppress()
	boundarysegment_section = (boundarysegment_header +
								Dict(OneOrMore(
									boundarysegment ))('boundarysegments') +
								section_end)

	sections = Each([vertex_section, simplex_section, boundarysegment_section])
	dgf_header = (Keyword('DGF') + linend).suppress()
	dgf = (dgf_header + Dict(sections) + OneOrMore( section_end ))('dgf')
	return dgf



# -*- coding: utf-8 -*-
"""
Created on Tue Oct  2 14:49:16 2012

@author: r_milk01
"""

import unittest
import os

from dune import supermodule
from dune.control import ModuleMissing, Dunecontrol

class TestControl(unittest.TestCase):

    def setUp(self):
        self.repo = supermodule.get_dune_stuff()

    def test_control(self):
        self.assertTrue(os.path.isdir(self.repo))
        ctrl = Dunecontrol.from_basedir(self.repo)
        self.assertIn('dune-common',
                      ctrl.dependencies('dune-stuff')['required'])
        with self.assertRaises(ModuleMissing):
            ctrl.dependencies('NONEXISTENT')
        ctrl.autogen('dune-common')
        ctrl.configure('dune-common')
        ctrl.make('dune-common')


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python

import os
import subprocess
import logging

class ModuleMissing(Exception):

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return 'Missing module: %s' % self.name


class Dunecontrol(object):

    @classmethod
    def from_basedir(cls, basedir):
        ctrl_path = os.path.join(basedir, 'dune-common', 'bin', 'dunecontrol')
        return cls(ctrl_path)

    def __init__(self,script_path):
        self._script_path = script_path
        self._base_dir = os.path.join(os.path.dirname(script_path), '../..')

    def _call(self, args):
        try:
            cl = [self._script_path] + args
        except:
            cl = [self._script_path] + args.split()
        return subprocess.check_output(cl, cwd=self._base_dir,
                                       stderr=subprocess.STDOUT)

    def printdeps(self, module):
        import pprint
        pprint.pprint(self.dependencies(module))

    def dependencies(self, module):
        try:
            out = self._call(['--module=%s'%module, 'printdeps']).split('\n')[1:]
        except subprocess.CalledProcessError, e:
            err_msg = 'ERROR: could not find module '
            idx = e.output.find(err_msg)
            if idx > -1:
                idx += len(err_msg)
                module = e.output[idx:e.output.find(',', idx)]
                raise ModuleMissing(module)
            raise e
        #output looks like: 'dune_common (required)'
        def cleanup(name):
            return m.split()[0].strip().replace('_', '-')
        required = [cleanup(m) for m in out if 'required' in m ]
        suggested = [cleanup(m) for m in out if 'suggested' in m ]
        return {'required':required, 'suggested': suggested}

    def configure(self,module=None):
        return self._call(['--module=%s'%module,'configure']).split()

    def autogen(self,module=None):
        return self._call(['--module=%s'%module,'autogen']).split()

    def make(self,module=None):
        return self._call(['--module=%s'%module,'make']).split()

if __name__ == "__main__":
    p = '/home/r_milk01/projekte/uni/dune/multiscale-super/dune-common/bin/dunecontrol'
    ctrl = Dunecontrol(p)
    print ctrl.printdeps('dune-stuff')




#!/usr/bin/python
"""
triangle2dgf.py (c) 2009 felix.albrecht@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

import sys
import math
import os
import time

## global defines

# about the correctness of the triangle files
triangle_nodefile_incorrect = 0
triangle_polyfile_incorrect = 0
triangle_elefile_incorrect = 0

# about the filenames

## done with global defines


## function definitions

# read vertices from a .node file generated by triangle
def read_vertices(filename) :
	print 'reading vertices from %s...' % (filename),
	global triangle_nodefile_incorrect
	list_of_vertices = []
	file = open(filename, 'r')
	number_of_vertices = 0
	number_of_vertices_set = 0
	vertex_number = 0
	for line in file.readlines():
		words = line.split()
		if words[0] != '#':
			if number_of_vertices_set == 0:
				if len(words) == 4:
					number_of_vertices = int(words[0])
					number_of_vertices_set += 1
			else:
				if vertex_number < number_of_vertices:
					if int(words[0]) == vertex_number:
						x = float(words[1])
						y = float(words[2])
						vertex = [x, y]
						list_of_vertices.append(vertex)
					else:
						if number_of_vertices != vertex_number:
							triangle_nodefile_incorrect += 1
					vertex_number += 1
	print '\t\tfound %i vertices' % (number_of_vertices)
	return list_of_vertices

# read faces from a .poly file, returns [ [ vertex_one, vertex_two, boundary_id ], ... ]
def read_faces(filename):
	print 'reading faces from %s...' % (filename),
	global triangle_polyfile_incorrect
	list_of_faces_with_ids = []
	file = open(filename, 'r')
	number_of_faces = 0
	number_of_faces_set = 0
	face_number = 0
	for line in file.readlines():
		words = line.split()
		if words[0] != '#':
			if number_of_faces_set == 0:
				if len( words ) == 2:
					number_of_faces = int(words[0])
					number_of_faces_set += 1
			else:
				if face_number < number_of_faces:
					if int(words[0]) == face_number:
						vertex_one = int(words[1])
						vertex_two = int(words[2])
						boundary_id = int(words[3])
						list_of_faces_with_ids.append([vertex_one, vertex_two,
														boundary_id])
					else:
						if number_of_faces != face_number:
							triangle_polyfile_incorrect += 1
					face_number += 1
	print '\t\tfound %i faces' % (number_of_faces)
	return list_of_faces_with_ids

# read simplices from a .ele file, returns [ [ vertex_one, vertex_two, vertex_three ], ... ]
def read_simplices(filename, faces):
	print 'reading simplices from %s...' % (filename),
	global triangle_elefile_incorrect
	list_of_simplices = []
	file = open(filename, 'r')
	number_of_simplices = 0
	number_of_simplices_set = 0
	simplex_number = 0
	for line in file.readlines():
		words = line.split()
		if words[0] != '#':
			if number_of_simplices_set == 0:
				if len(words) == 3:
					number_of_simplices = int(words[0])
					number_of_simplices_set += 1
			else:
				if simplex_number < number_of_simplices:
					if int(words[0]) == simplex_number:
						simplex = []
						for i in range(1, 4):
							simplex.append(int(words[i]))
						list_of_simplices.append(simplex)
					else:
						if simplex_number != number_of_simplices:
							triangle_elefile_incorrect += 1
					simplex_number += 1
	print '\t\tfound %i simplices' % (simplex_number)
	return list_of_simplices


# write the dgf header into the dgf file
def write_dgf_header(file):
	file.write('DGF\t\t\t\t\t\t%s %s, generated by triangle2dgf.py on %s \n' %
				('%', file.name, time.strftime('%Y/%m/%d %H:%M:%S')))
	#file.write( '% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n' )
	#file.write( '%s %s\n' %( '%', file.name ) )
	#file.write( '%s written by triangle2dgf.py on %s\n' %( '%', time.strftime('%Y/%m/%d %H:%M:%S') ) )
	#file.write( '% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n' )


# write vertices to dgf file
def write_vertices(vertices,file):
	print 'writing vertices to %s...' % (file.name),
	file.write('VERTEX\t\t\t\t\t% the vertices of the grid\n')
	if triangle_nodefile_incorrect == 0:
		vertex_number = 0
		for vertex in vertices:
			file.write('%f\t%f\t%s vertex %i\n' %
						( vertex[0], vertex[1], '%', vertex_number))
			vertex_number += 1
		file.write('#\n')
		print '\t\t%i vertices written' % (vertex_number)
	else :
		print 'error: write_vertices() not implemented for nonconforming .node file'
		sys.exit()

# write simplices to dgf file
def write_simplices(simplices, file) :
	print 'writing simplices to %s...' % (file.name),
	file.write('SIMPLEX\t\t\t\t\t% the simplices of the grid\n')
	if triangle_elefile_incorrect == 0:
		simplex_number = 0
		for simplex in simplices:
			file.write('%i\t%i\t%i\t\t\t\t%s simplex %i, consisting of vertices %i, %i and %i\n' %( simplex[ 0 ], simplex[ 1 ], simplex[ 2 ], '%', simplex_number, simplex[ 0 ], simplex[ 1 ], simplex[ 2 ] ) )
			simplex_number += 1
		file.write( '#\n' )
		print '\t\t%i simplices written' %( simplex_number )
	else :
		print 'error: write_simplices() not implemented for nonconforming .ele file'
		sys.exit()

# write boundary segments to dgf file
def write_boundary_segments( faces_with_ids, file ) :
	print 'writing boundary segments to %s...' %( file.name ),
	file.write( 'BOUNDARYSEGMENTS\t\t% the boundary segments of the grid\n' )
	if triangle_polyfile_incorrect == 0 :
		boundary_segment_number = 0
		for face_with_id in faces_with_ids :
			vertex_one = face_with_id[ 0 ]
			vertex_two = face_with_id[ 1 ]
			id = face_with_id[ 2 ]
			file.write( '%i\t%i\t%i\t\t\t\t%s boundary ID %i between vertices %i and %i\n' %( id, vertex_one, vertex_two, '%', id, vertex_one, vertex_two ) )
			boundary_segment_number += 1
		file.write( '#\n' )
		file.write( 'BOUNDARYDOMAIN\n' )
		file.write( 'default 1\n' )
		file.write( '#\n')
		print '\t%i boundary segments written' %( boundary_segment_number )
	else :
		print 'error: write_boundary_segments() not implemented for nonconforming .poly file'
		sys.exit()

## done with function definitions


def main():
	filename_prefix = sys.argv[ 1 ]
	dgf_filename = '%s.dgf' %( filename_prefix )
	nodefile_filename = filename_prefix + '.node'
	polyfile_filename = filename_prefix + '.poly'
	elefile_filename = filename_prefix + '.ele'

	dgf_file = open( dgf_filename, 'w' )

	vertices = read_vertices( nodefile_filename )
	if triangle_nodefile_incorrect != 0 :
		print 'error: the triangle nodefile is incorrect'
		sys.exit()

	faces_with_boundary_ids = read_faces( polyfile_filename )
	if triangle_polyfile_incorrect != 0 :
		print 'error: the triangle polyfile is incorrect'
		sys.exit()
		
	simplices = read_simplices( elefile_filename, faces_with_boundary_ids )
	if triangle_nodefile_incorrect != 0 :
		print 'error: the triangle nodefile is incorrect'
		sys.exit()

	write_dgf_header( dgf_file )
	write_vertices( vertices, dgf_file )
	write_simplices( simplices, dgf_file )
	write_boundary_segments( faces_with_boundary_ids, dgf_file )



#!/usr/env/python
# -*- coding: utf-8 -*-
"""
gridhelper.py (c) 2009 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

class DimensionIncompatibleException(Exception):
	pass

class ImpossibleException(Exception):
	pass

import math, copy
from euclid import Vector3, Matrix4

class PLCPointList:
	"""static var numbering all vertices"""
	global_vertices = []

	def __init__(self, dim ):
		self.dim = dim
		self.verts = dict()
		self.attribs = dict()
		#set to -1 so we get a 0-based index for verts dict
		self.duplicate_count = -1
		#alias -> real vertex id mapping
		self.aliases = dict()
		self.duplicates = []

	def appendVert(self,x,c=Vector3(0,0,0)):
		assert isinstance( c, Vector3 )
		glob_idx = len(PLCPointList.global_vertices) + len(self.aliases) 
		#PLCPointList.global_vertices.append(x)
		if not x in PLCPointList.global_vertices:
			PLCPointList.global_vertices.append(x)
		else:
			assert x not in self.duplicates
			self.duplicates.append( x )
			x_id = find_key(self.verts, x)
			self.aliases[glob_idx] = x_id
			assert self.verts[x_id] == x, '%d -- %d | %s -- %s'%(glob_idx,x_id, x, self.verts[x_id] )
		
		self.verts[glob_idx] = x
		self.attribs[glob_idx] = c
		return glob_idx

	def simpleString(self):
		ret = ''
		for idx,p in self.verts.iteritems():
			ret += 'Vertix %4d \t%s\n'% (idx,str(p))
		return ret

	def __len__(self):
		return len(self.verts)

	def unalias(self,idx):
		if self.aliases.has_key(idx):
			return self.aliases[idx]
		else:
			return idx
		
class Simplex:
	def __init__(self,v1,v2,v3):
		assert isinstance( v1, int )
		self.v1 = v1
		assert isinstance( v2, int )
		self.v2 = v2
		assert isinstance( v3, int )
		self.v3 = v3

	def __repr__(self):
		return 'Simplex (%4d,%4d,%4d)'%(self.v1,self.v2,self.v3)

class FanningSimplexList:
	def __init__(self,center_idx,bid):
		self.center_idx = center_idx
		self.simplices 	= []
		self.boundaryId = bid
		self.vertex_idx = []

	def addVertex(self,v_idx):
		assert v_idx >= 0
		self.vertex_idx.append( v_idx )
		if len(self.vertex_idx) > 1:
			self.simplices.append( Simplex( self.center_idx, self.vertex_idx[-1], self.vertex_idx[-2]  ) )

	def close(self):
		"""finish last simplex"""
		self.simplices.append( Simplex( self.center_idx, self.vertex_idx[-1], self.vertex_idx[0]  ) )

	def __repr__(self):
		ret = 'FanningSimplexList  for boundary ID %d\n'%(self.boundaryId)
		i = 0
		for s in self.simplices:
			ret += '%4d %s\n'%(i,s)
			i += 1
		return ret

	def __str__(self):
		return self.__repr__()

class InbetweenRing:
	def __init__(self):
		self.vertex_idx = []

	def addVertex(self,v_idx):
		self.vertex_idx.append( v_idx )

	def __repr__(self):
		ret = 'InbetweenRing \n'
		i = 0
		for s in self.vertex_idx:
			ret += 'Vertex %4d %s\n'%(i,s)
			i += 1
		return ret

	def __str__(self):
		return self.__repr__()


class FullGrid:
	def __init__(self,f1,default_Bid):
		self.fans 					= [f1]
		self.default_Bid 			= default_Bid
		self.connecting_simplices 	= []
		self.rings					= []

	def connect(self,new_f):
		f1 = f2 = None
		if isinstance( new_f, InbetweenRing ):
			print 'connecting InbetweenRing\n'
			self.rings.append( new_f )
			if len(self.rings) > 1:
				f1 = self.rings[-2]
			else:
				f1 = self.fans[-1]
		elif isinstance( new_f, BoundarySurface ):
			print 'connecting BoundarySurface\n'
			self.fans.append( new_f )
			if len(self.rings) > 0:
				f1 = self.rings[-1]
			else:
				f1 = self.fans[-2]
		f2 = new_f
		if len(f1.vertex_idx) !=  len(f2.vertex_idx):
			raise DimensionIncompatibleException()
		b_len = len(f1.vertex_idx)
		for i in range ( 0, b_len  ):
			self.connecting_simplices.append( \
				Simplex(	f1.vertex_idx[i],
							f2.vertex_idx[i-1], \
							f1.vertex_idx[i-1] ) )
			self.connecting_simplices.append( \
				Simplex(	f1.vertex_idx[i],
							f2.vertex_idx[i], \
							f2.vertex_idx[i-1] ) )
	def __str__(self):
		ret = 25*'=' + 'GRID'+ 25*'=' + '\nconnecting simplices %d\n'%(len(self.connecting_simplices))
		for s in self.connecting_simplices:
			ret += str(s) + '\n'
		ret += 'FanningSimplexLists:\n'
		for f in self.fans:
			ret += str(f) + '\n'
		ret += 'InbetweenRings:\n'
		for r in self.rings:
			ret += str(r) + '\n'
		ret += 25*'=' + 'GRID'+ 25*'='
		return ret

	def outputPLC(self, fn, args, options ):
		out = None
		if not fn.endswith( '.smesh' ):
			fn += '.smesh'
		try:
			out = open(fn,'w')
		except:
			raise ImpossibleException()
		out.write( '# commandline: %s\n'%' '.join( args ) )
		out.write( '# options: %s\n'%options )
		out.write( '%d 3 0 %d\n'%(len(PLCPointList.global_vertices),3) )#3 bids
		out.write( '# all vertices\n#\n' )
		cVert = 0
		for v in PLCPointList.global_vertices:
				out.write( '%d %f %f %f\n'%(cVert,v.x,v.y,v.z) )
				cVert += 1
		boundary_segment_count = len(self.connecting_simplices)
		for f in self.fans:
			boundary_segment_count += len(f.simplices)
		out.write( '\n# number of facets (= number of triangles), border marker\n#\n%d 1\n'%(boundary_segment_count) )
		out.write( '# all faces\n#\n' )
		cBseg = 0
		for f in self.fans:
			assert isinstance( f, BoundarySurface )
			for b1 in f.simplices:
				assert isinstance( b1, Simplex )
				out.write( '%d %d %d %d %d\n'%(3,b1.v1,b1.v2,b1.v3,f.boundaryId)  )
				cBseg += 1
		for c in self.connecting_simplices:
			out.write( '%d %d %d %d %d\n'%(3,c.v1,c.v2,c.v3,self.default_Bid)  )
			cBseg += 1
		out.write( '%d\n'%(0))

class BoundarySurface:
	def __init__(self, bidMapper, mid, first, count_inner_rings, count_spokes, bid, flip_face_def=False ):
		self.color				= bidMapper.getColor(bid)
		self.mid 				= mid
		self.first 				= first
		self.count_inner_rings 	= count_inner_rings
		self.count_spokes 		= count_spokes
		self.boundaryId 		= bid
		self.alpha				= math.radians( 360. / (self.count_spokes ) )
		self.alpha_half			= self.alpha / 2.0

		self.simplices				= []	
		self.outer_vertices_idx 	= []
		self.inner_vertices_idx 	= []
		self.vertices 				= PLCPointList( 3 )
		self.spokes					= []
		self.mid_idx 				= self.vertices.appendVert( self.mid, self.color )
		
		rot_mat = Matrix4.new_rotatez( self.alpha )
		
		L = Vector3(self.first.x, self.first.y, self.first.z)
		z_offset = Vector3(0, 0, self.first.z)
		for i in range( 0, self.count_spokes  ):
			if i != 0:
				L -= z_offset
				L = rot_mat * L
				L += z_offset
			L_idx = self.vertices.appendVert( L, self.color )
			self.outer_vertices_idx.append( L_idx )
			spoke_direction = self.mid - L 
			assert self.mid.z == L.z
			spoke_vertex_idx = []
			spoke_vertex_idx.append( L_idx )
			spoke_add  = spoke_direction / float( self.count_inner_rings )
			for i in range( 1, self.count_inner_rings ):
				p = L + ( i * spoke_add )
				spoke_vertex_idx.append( self.vertices.appendVert( p, self.color ) )
			self.inner_vertices_idx.append( spoke_vertex_idx[-1] )
			self.spokes.append( spoke_vertex_idx )

		for i in range( 0, self.count_spokes ):
			current_spoke = self.spokes[i-1]
			left_spoke = self.spokes[i-2]
			right_spoke = self.spokes[i]
			for j in range( 0, self.count_inner_rings - 1 ):
				#print 'current spoke j,j+1, left spoke j: %f - %f - %f '%(current_spoke[j], current_spoke[j+1], left_spoke[j])
				if flip_face_def:
					self.simplices.append( Simplex( current_spoke[j], current_spoke[j+1], left_spoke[j]  ) )
					self.simplices.append( Simplex( right_spoke[j+1], current_spoke[j+1], current_spoke[j] ) )
				else:
					self.simplices.append( Simplex( left_spoke[j], current_spoke[j+1], current_spoke[j] ) )
					self.simplices.append( Simplex( current_spoke[j], current_spoke[j+1], right_spoke[j+1] ) )

		for i in range( 0, len(self.inner_vertices_idx)  ):
			if flip_face_def:
				self.simplices.append( Simplex( self.inner_vertices_idx[i-1], self.inner_vertices_idx[i], self.mid_idx ) )
			else:
				self.simplices.append( Simplex( self.mid_idx, self.inner_vertices_idx[i],self.inner_vertices_idx[i-1] ) )

		#this is exposed to FullGrid
		self.vertex_idx = self.outer_vertices_idx

class adaptVec(list):
	def __init__(s,v):
		s.append(v.x)
		s.append(v.y)
		s.append(v.z)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
mesh.py (c) 2009 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

from OpenGL.GL import (glBegin, glVertex3fv, glPushMatrix, glTranslatef,
	glPopMatrix, glEndList, glCallList, glColor4f, glNewList, GL_COMPILE,
	GL_LINE_STRIP, GL_LINES, glVertex, glEnd, glGenLists, glLineWidth,
	GLdouble, GLint, glEnd)
from OpenGL.GLU import (gluNewQuadric, gluSphere)
from dune.mesh.util.euclid import Vector3
from dune.mesh.util.gridhelper import adaptVec
import math

class BoundingVolume:
	def __init__(self,mesh):
		self.outline_color = (1, 1, 1)
		vertices = mesh.vertex_list.getVertices()
		minV = Vector3()
		maxV = Vector3()
		vertices.sort( lambda x, y: cmp(y.x, x.x ) )
		minV.x = vertices[0].x
		maxV.x = vertices[-1].x
		vertices.sort( lambda x, y: cmp(y.y, x.y ) )
		minV.y = vertices[0].y
		maxV.y = vertices[-1].y
		vertices.sort( lambda x, y: cmp(y.z, x.z ) )
		minV.z = vertices[0].z
		maxV.z = vertices[-1].z

		self.points = []
		for i in range(8):
			self.points.append(Vector3())
		for i in range(2):
			for j in range(2):
				self.points[int('%d%d%d'%(0,i,j),2)].x = minV.x
				self.points[int('%d%d%d'%(1,i,j),2)].x = maxV.x
				self.points[int('%d%d%d'%(i,0,j),2)].y = minV.y
				self.points[int('%d%d%d'%(i,1,j),2)].y = maxV.y
				self.points[int('%d%d%d'%(i,j,0),2)].z = minV.z
				self.points[int('%d%d%d'%(i,j,1),2)].z = maxV.z

		self.center = Vector3()
		for p in self.points:
			self.center += p
		self.center /= float(8)
		self.dl = glGenLists(1)
		glNewList(self.dl,GL_COMPILE)
		glLineWidth(5)
		c = self.outline_color
		glColor4f(c[0],c[1],c[2],0.2)
		glBegin(GL_LINE_STRIP)
		for v in self.points:
			glVertex(v())
		glEnd()
		c = self.points
		glBegin(GL_LINES)

		glVertex(c[0]())
		glVertex(c[2]())
		glVertex(c[6]())
		glVertex(c[4]())

		glVertex(c[1]())
		glVertex(c[3]())
		glVertex(c[7]())
		glVertex(c[5]())

		glVertex(c[0]())
		glVertex(c[1]())
		glVertex(c[3]())
		glVertex(c[2]())

		glVertex(c[2]())
		glVertex(c[3]())
		glVertex(c[7]())
		glVertex(c[6]())

		glVertex(c[7]())
		glVertex(c[6]())
		glVertex(c[4]())
		glVertex(c[5]())

		glVertex(c[0]())
		glVertex(c[1]())
		glVertex(c[5]())
		glVertex(c[4]())

		glEnd()
		glPushMatrix(  )
		glTranslatef(self.center.x,self.center.y,self.center.z)
		q = gluNewQuadric()
		gluSphere( q, GLdouble(0.25), GLint(10), GLint(10) )
		glPopMatrix(  )
		glEndList()

	def draw(self):
		glCallList(self.dl)

	def __repr__(self):
		ret = 'bounding box\n'
		for p in self.points:
			ret += '%s\n'%str(p)
		ret += '---\n'
		return ret

	def minViewDistance(self, viewangle=45.0):
		r = abs(self.center - self.points[0] )
		t = math.radians( (180 - viewangle)/ 2.0 )
		return 1.12 * r * math.tan(t)



#!/usr/bin/env python
"""
viewer.py (c) 2009 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

from PyQt4 import QtGui
from widgets import MeshViewer

usage = 'usage %s FILENAME'

def main(argv):
	app = QtGui.QApplication(['MeshViewer'])
	if len(argv) < 2 :
		print usage % argv[0]
	else:
		filename = argv[1]
		window = MeshViewer(filename)
		window.show()
		app.exec_()




#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
grammar.py (c) 2011 rene.milk@uni-muenster.de,felix.albrecht@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""


import sys, math, os, time
from optparse import OptionParser

# calculate points on outer rectangle
# beginning bottom left
# rather trivial
# returns list of points in 2d
# [ [ point_x, point_y ], id_of_face_starting_clockwise_at_this_point ]
def generate_rectangle( length_x, length_y, ids ) :
	rectangle = []
	rectangle.append( [ [ 0.0, 0.0 ], ids[0] ] )
	rectangle.append( [ [ length_x, 0.0 ], ids[1] ] )
	rectangle.append( [ [ length_x, length_y ], ids[2] ] )
	rectangle.append( [ [ 0.0, length_y ], ids[3] ] )
	return rectangle

# write to trianlge
def write_to_triangle( rectangle, triangle_filename ) :
	file = open( triangle_filename, 'w' )
	# header
	file.write( '# #########################################################\n' )
	file.write( '# %s\n' %( triangle_filename ) )
	file.write( '# written by generateCube.py on %s\n' %( time.strftime('%Y/%m/%d %H:%M:%S') ) )
	total_number_of_vertices =len( rectangle )
	total_number_of_faces =len( rectangle )
	file.write( '# we have \t%i vertices\n' %( total_number_of_vertices ) )
	file.write( '#\t\t%i faces\n' %( total_number_of_faces ) )
	file.write( '#\t\t%i hole\n' %(1) )
	file.write( '# #########################################################\n' )
	file.write( '#\n' )
	# write the vertices
	file.write( '# these are the %i vertices\n' %( total_number_of_vertices ) )
	file.write( '%i 2 0 0\n' %( total_number_of_vertices ) )
	vertex_number = 0
	# and the outer rectangle
	for point_with_id_on_rectangle in rectangle :
		point_on_rectangle = point_with_id_on_rectangle[ 0 ]
		x = point_on_rectangle[ 0 ]
		y = point_on_rectangle[ 1 ]
		file.write( '\t%i\t%f\t%f\n' %( vertex_number, x, y ) )
		vertex_number += 1
	file.write( '# these were the %i vertices\n' %( total_number_of_vertices ) )
	file.write( '#\n' )
	# write the faces with boundary id
	file.write( '# these are the %i faces\n' %( total_number_of_faces ) )
	file.write( '%i 1\n' %(total_number_of_faces) )
	face_number = 0
	# of the outer rectangle
	for point_with_id_on_rectangle in rectangle :
		id = point_with_id_on_rectangle[ 1 ]
		file.write( '\t%i\t%i\t%i\t%i\n' %( face_number, face_number % len( rectangle ), ( face_number +1 ) % len( rectangle ), id ) )
		face_number += 1
	file.write( '# these were the %i faces\n' %( total_number_of_faces ) )
	file.write( '#\n' )
	# write the holes
	file.write( '# this is the hole\n' )
	file.write( '0\n' )
	# ending
	file.write( '# #########################################################\n' )
	file.write( '# end of file %s\n' %( triangle_filename ) )
	file.write( '# written by generateCube.py on %s\n' %( time.strftime('%Y/%m/%d %H:%M:%S') ) )
	file.write( '# #########################################################\n' )

## done with function definitions


def generate():
	parser = OptionParser()
	parser.add_option("-x", "--length_x", dest="length_x", default=1.,
					help="rectangle length_x", type='float')
	parser.add_option("-y", "--length_y", dest="length_y", default=1.,
					help="rectangle length_y", type='float')
	parser.add_option("-f", "--filename", dest="filename", default='cube_in_2d.poly',
					help="output filename", type='string')
	(options, args) = parser.parse_args()

	# about the outer rectangle
	rectangle_length_x = options.length_x
	rectangle_length_y = options.length_y

	# about the boundary ids
	id_of_bottom_rectangle_faces = 2
	id_of_right_rectangle_faces = 3
	id_of_top_rectangle_faces = 4
	id_of_left_rectangle_faces = 5

	# about the files to be written
	triangle_filename = options.filename

	points_on_rectangle = generate_rectangle( rectangle_length_x, rectangle_length_y, [ id_of_bottom_rectangle_faces, id_of_right_rectangle_faces, id_of_top_rectangle_faces, id_of_left_rectangle_faces ] )
	write_to_triangle( points_on_rectangle, triangle_filename )



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
gridhelper.py (c) 2009 rene.milk@uni-muenster.de,felix.albrecht@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

import sys,math,os,time

## global defines

# about the correctness of the triangle files
triangle_nodefile_incorrect = 0
triangle_polyfile_incorrect = 0
triangle_elefile_incorrect = 0

def assert_line(line):
	for el in line:
		assert el >= -1,'line has element with negative index: %s'%' '.join(line)

# write the dgf header into the dgf file
def write_dgf_header( file ) :
	file.write( 'DGF\t\t\t\t\t\t%s %s, generated by %s on %s \n' %( '%', file.name, sys.argv[0],time.strftime('%Y/%m/%d %H:%M:%S') ) )
	#file.write( '% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n' )
	#file.write( '%s %s\n' %( '%', file.name ) )
	#file.write( '%s written by triangle2dgf.py on %s\n' %( '%', time.strftime('%Y/%m/%d %H:%M:%S') ) )
	#file.write( '% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n' )

# write vertices to dgf file
def write_vertices( file, node_file_name ) :
	print 'writing vertices to %s...' %( file.name ),
	file.write( 'VERTEX\t\t\t\t\t% the vertices of the grid\n' )
	with open( node_file_name, 'r' ) as node_file:
		vertex_number = 0
		for line in node_file.readlines():
			if vertex_number != 0:
				if line.startswith( '#' ):
					continue
				line = [ float(el) for el in line.split() ]
				assert len(line) > 3
				file.write( '%f\t%f\t%f\t%% vertex %i\n' %( line[1], line[2], line[3], line[0] - 1 ) )
			vertex_number += 1
		file.write( '#\n' )

# write simplices to dgf file
def write_simplices( file, ele_file_name ) :
	print 'writing simplices to %s...' %( file.name ),
	file.write( 'SIMPLEX\t\t\t\t\t% the simplices of the grid\n' )
	with open( ele_file_name, 'r' ) as ele_file:
		vertex_number = 0
		for line in ele_file.readlines():
			if vertex_number != 0:
				if line.startswith( '#' ):
					continue
				offset = 0
				line = [ int(el) + offset for el in line.split() ]
				assert len(line) > 4 , line
				assert_line(line)
				file.write( '%i\t%i\t%i\t%i\t%% simplex %i\n' %( line[1] , line[2], line[3], line[4], line[0]))
			vertex_number += 1
	file.write( '#\n' )

# write boundary segments to dgf file
def write_boundary_segments( file, face_file_name ) :
	print 'writing boundary segments to %s...' %( file.name ),
	file.write( 'BOUNDARYSEGMENTS\t\t% the boundary segments of the grid\n' )
	with open( face_file_name, 'r' ) as face_file:
		vertex_number = 0
		for line in face_file.readlines():
			if vertex_number != 0:
				if line.startswith( '#' ):
					continue
				line = [ int(el) for el in line.split() ]
				assert len(line) > 4 , line
				file.write( '%i\t%i\t%i\t%i\t%% segment %i\n' %( line[4], line[1] - 1 , line[ 2 ] - 1, line[3] - 1, line[0] - 1 ))
			vertex_number += 1
	file.write( '#\n' )
	file.write( '#\n' )
	file.write( 'BOUNDARYDOMAIN\n' )
	file.write( 'default 1\n' )
	file.write( '#\n')

## done with function definitions


def main():
	filename_prefix = sys.argv[ 1 ]
	dgf_filename = '%s.dgf' %( filename_prefix )
	nodefile_filename = filename_prefix + '.node'
	facefile_filename = filename_prefix + '.face'
	elefile_filename = filename_prefix + '.ele'

	with open( dgf_filename, 'w' ) as dgf_file:
		write_dgf_header( dgf_file )
		write_vertices( dgf_file, nodefile_filename )
		write_simplices( dgf_file, elefile_filename )
		write_boundary_segments( dgf_file, facefile_filename )



import sys
from distutils.core import setup

setup(
    name = 'pyDune',
    version = '0.1.4',
    author = 'Rene Milk',
    author_email = 'rene.milk@uni-muenster.de',
    packages = ['dune', 'dune.mesh', 'dune.mesh.gui', 'dune.mesh.dgf', 'dune.mesh.smesh',
        'dune.mesh.util', 'dune.mesh.converter', 'dune.mesh.generators'],
    scripts = ['bin/%s'%n for n in ['mesh-convert', 'mesh-generate', 'mesh-viewer', 'tetgen2dgf', 'triangle2dgf', 'dune-supermodule'] ],
    url = 'http://users.dune-project.org/projects/pydune',
    description = 'assorrted DUNE related scripts and utilities',
    long_description = open('README.txt').read(),
    # running `setup.py sdist' gives a warning about this, but still
    # install_requires is the only thing that works with pip/easy_install...
    # we do not list pyqt here since pip can't seem to install it
    install_requires = ['matplotlib', 'pyparsing', 'nose-cov', 'nose', 'nosehtmloutput', 'rednose', 'GitPython'],
    classifiers = ['Development Status :: 4 - Beta',
        'Environment :: X11 Applications :: Qt',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Visualization'],
    license = 'LICENSE.txt'
)



# -*- coding: utf-8 -*-
"""
Created on Tue Oct  2 14:00:46 2012

@author: r_milk01
"""

import unittest
import os

from dune import supermodule

class TestSuper(unittest.TestCase):


    def test_create(self):
        stuff_dir = supermodule.get_dune_stuff()
        self.assertFalse(os.path.isdir(stuff_dir))


if __name__ == '__main__':
    unittest.main()


"""
grammar.py (c) 2011 rene.milk@uni-muenster.de
Licence: WTFPLv2, see LICENSE.txt
"""

import math

class IFunctor:
	def scale(self, x):
		raise NotImplementedException()
	
class HyperboleFunctorZ(IFunctor):
	def __init__(self,tube_length,fac,additive):
		self.tube_length	= tube_length
		self.fac 			= fac
		self.additive		= additive
	
	def scale(self, x):
		mult = self.fac
		#if x.z != self.tube_length:
		mult *= (self.tube_length - x.z) / float( self.tube_length + x.z ) + self.additive
		x.x *= mult
		x.y *= mult
		return x
	
class IdentityFunctor(IFunctor):
	def scale(self, x):
		return x
	